#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "dnsmasq.h"

#ifdef	DOME_LED_FUTURE
#define BLOCK_IP_ADDR_STR	"208.67.219.130"
#define DNS_HEADER_LEN	12

#ifdef DOME_DEBUG  
#   define DBG(fmt, args...)   printf("[DOME_DNS] "fmt, ##args)                                                              
#else
#   define DBG(fmt, args...)   do{ }while(0)
#endif

extern int config_match(char *name, char *match);
extern char *config_get(char *name);

typedef struct _dns_domain_t {
	char name[255];
	u32 len;
}dns_domain_t;

typedef struct _dns_header_t {
	u16 id;
	u16 flags;
	u16 ques;
	u16 ans_rrs;
	u16 auth_rrs;
	u16 addi_rrs;
	/*query*/
} dns_header_t;

typedef struct _dns_answer_t {
	/*answer*/
	u16 ans_offset;
	u16 ans_type;
	u16 ans_class;
	u32 ans_ttl;
	u16 ans_len;
	u32 ans_addr;   /*suppose for ipv4 */
} __attribute__((__packed__)) dns_answer_t;  

void parse_dns_domain(u8 *pkg, dns_domain_t *dom)
{
	char name[256] = {0};
	char *tmp, *p;
	int i, len;
	
	if (!pkg || !dom)
		return;

	tmp = pkg;
    p = name;

    len = pkg[0];
    for (i = 0; *tmp != '\0'; i++, tmp++) {
		if (i > 255)
			break;
        if (i == len + 1) {
            len = len + *tmp + 1;
            *p++ = '.';
        } else {
            if (i != 0)
             *p++ = *tmp;
        }
    }

    *p = '\0';

	strcpy(dom->name, name);
	
	dom->len = strlen(name) + 2 + 2 + 2;
}

u32 extract_dns_answer(u8 *pkg)
{
	u8 *tmp;	
	dns_header_t *hdr_ptr;	
	dns_domain_t dom;
	dns_answer_t *ans;
	if (!pkg)
		return -1;

	hdr_ptr = (dns_header_t *)pkg;

	if (hdr_ptr->flags != htons(0x8180)) {
		DBG("Not query response skip... flags 0x%x\n", hdr_ptr->flags);
		return -1;
	}
	if (hdr_ptr->ques != htons(0x0001) || hdr_ptr->ans_rrs != htons(0x0001) ) {
		/*Now only parse the simple situation*/
		DBG("more than 1 ques and answ skip...ques = 0x%x ans_rrs = 0x%x\n",
						hdr_ptr->ques, hdr_ptr->ans_rrs);
		return -1;		
	}

	tmp = pkg + DNS_HEADER_LEN;

	parse_dns_domain(tmp, &dom);
	DBG("domain %s len %d\n", dom.name, dom.len);
	
	tmp = pkg + DNS_HEADER_LEN + dom.len;

	ans = (dns_answer_t *)tmp;
	
	/*pkg_dump(tmp, 16);*/
	DBG("offset 0x%x\n", ans->ans_offset);	
	DBG("type 0x%x\n", ans->ans_type);		
	DBG("class 0x%x\n", ans->ans_class);	
	DBG("ttl 0x%x\n", ans->ans_ttl);	
	DBG("len 0x%x\n", ans->ans_len);
	DBG("ip address 0x%x\n", ans->ans_addr);

	return ans->ans_addr;
}

int dome_led_blink_for_pc(u8 *pkg_reply)
{
	struct in_addr inaddr;	
	char *cfg_ip;
	char *p;
	u32 ret;
	
	if (config_match("ParentalControl", "0") || !config_match("dome_led_func", "2"))
		return -1;

	ret = extract_dns_answer(pkg_reply);

	if (ret == -1)
		return -1;

	/*Can this method be used in either Big or Little endian CPU?*/
	inaddr.s_addr = ret;
	
	p = inet_ntoa(inaddr);
	DBG("dns server return ip = %s\n", p);
	
	cfg_ip = config_get("dome_block_ip");
	if (!strcmp("", cfg_ip))
		cfg_ip = BLOCK_IP_ADDR_STR;

	if (strcmp(cfg_ip, BLOCK_IP_ADDR_STR))
		printf("[DOME_DNS] ******Using user configed block ip %s\n", cfg_ip);

	DBG("Configed block ip = %s\n", cfg_ip);
	if (p && !strcmp(p, cfg_ip))
		return 1;

	return 0;
}
#endif //DOME_LED_FUTURE
